from typing import List, Dict, Any, Tuple
from .en import detect_dates_en
from .pl import detect_dates_pl
from .ru import detect_dates_ru
from src.detections import Detection, DateDetection
from src.detectors.interface import Detector


class DateDetector(Detector):
    def __init__(self, language: str = "pl") -> None:
        self._language = language

    def detect(
        self, text: str, annotations: Dict[str, List[Tuple[int, int, Any]]]
    ) -> List[Tuple[int, int, DateDetection]]:
        return find_dates(text, self._language)


def find_dates(text: str, language: str = "pl") -> List[Tuple[int, int, DateDetection]]:
    """
    Finds dates in the text.
    :param text: the text to be searched
    :type text: str
    :param language: the language of the text
    :type language: str
    :return: a list of tuples containing (start, end, annotation)
    :rtype: List[Tuple[int, int, Annotation]]
    """

    language_processors = {
        "en": detect_dates_en,
        "pl": detect_dates_pl,
        "ru": detect_dates_ru,
    }

    return language_processors.get(language, detect_dates_en)(text)
