import regex as re
from typing import List, Tuple
from src.detections import DateDetection

from src.detectors.date.utils import _parse_date_to_format

EN_DATES_REGEX = re.compile(
    r"\b(?P<day_or_month_year>"
    r"(?P<day_month1>[0-3]?\d)(?P<punct1>[ \t\-\./,]{1,2})"
    r"(?P<day_month2>[0-3]?\d)(?P<punct2>[ \t\-\./,]{1,2})"
    r"(?P<year1>\d{4}|\d{2}))\b|"
    r"\b(?P<year_month_or_day>(?P<year2>\d{4}|\d{2})"
    r"(?P<punct3>[ \t\-\./,]{1,2})(?P<day_month3>[0-3]?\d)"
    r"(?P<punct4>[ \t\-\./,]{1,2})(?P<day_month4>[0-3]?\d))\b|"
    r"(?P<month_in_words>"
    r"(?:(?P<day1>[0-3]?\d)(?P<punct5>[ \t\-\./,]{0,2}))?"
    r"\b(?P<month>Jan(?:|uary)|Feb(?:|ruary)|Mar(?:|ch)|"
    r"Apr(?:|il)|May|Jun(?:|e)|Jul(?:|y)|Aug(?:|ust)|Sep(?:|tember)"
    r"|Oct(?:|ober)|Nov(?:|ember)|Dec(?:|ember))\b"
    r"(?:(?P<punct7>[ \t\-\./,]{0,2})(?P<day2>[0-3]?\d))?"
    r"(?:(?P<punct6>[ \t\-\./,]{1,2})(?P<year3>\d{4}|\d{2}))?"
    r"(?<!\b(jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)\b))",
    re.I,
)


def detect_dates_en(text: str) -> List[Tuple[int, int, DateDetection]]:
    """
    Detects English dates in the text.
    :param text: the text to be searched
    :type text: str
    :return: a list of tuples containing (start, end, annotation)
    :rtype: List[Tuple[int, int, DateAnnotation]]
    """
    matches = EN_DATES_REGEX.finditer(text)
    dates = []
    for match in matches:
        format = _parse_date_to_format(match.groupdict())
        dates.append((match.start(), match.end(), DateDetection(format)))
    return dates
